@@ -41,7 +41,7 @@
 #define PACKAGE "lame"
 
 /* Version number of package */
-#define VERSION "3.92"
+#define VERSION "3.95"
 
 /* Define if compiler has function prototypes */
 #define PROTOTYPES 1
@@ -52,6 +52,9 @@
 /* IEEE754 compatible machine */
 #define TAKEHIRO_IEEE754_HACK 1
 
+/* faster log implementation with less but enough precission */
+#define USE_FAST_LOG 1
+
 #define HAVE_STRCHR
 #define HAVE_MEMCPY
 
@@ -74,6 +77,11 @@
 #define uint16_t unsigned short
 #define uint32_t unsigned int
 #define uint64_t unsigned long long
+
+#define int8_t signed char
+#define int16_t signed short
+#define int32_t signed int
+#define int64_t signed long long
 #endif
 
 typedef long double ieee854_float80_t;
@@ -81,7 +89,7 @@ typedef double      ieee754_float64_t;
 typedef float       ieee754_float32_t;
 
 #ifdef LAME_ACM
-// memory hacking for driver purposes
+/* memory hacking for driver purposes */
 #define calloc(x,y) acm_Calloc(x,y)
 #define free(x)     acm_Free(x)
 #define malloc(x)   acm_Malloc(x)
@@ -90,6 +98,7 @@ typedef float       ieee754_float32_t;
 void *acm_Calloc( size_t num, size_t size );
 void *acm_Malloc( size_t size );
 void acm_Free( void * mem);
-#endif // LAME_ACM
+#endif /* LAME_ACM */
 
 #define LAME_LIBRARY_BUILD
+
