@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: main.c,v 1.76 2002/10/15 09:33:42 aleidinger Exp $ */
+/* $Id: main.c,v 1.85 2003/12/21 18:41:33 bouvigne Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -48,6 +48,11 @@ char   *strchr(), *strrchr();
 # include <fcntl.h>
 #endif
 
+#ifdef __sun__
+/* woraround for SunOS 4.x, it has SEEK_* defined here */
+#include <unistd.h>
+#endif
+
 #if defined(_WIN32)
 # include <windows.h>
 #endif
@@ -191,7 +196,7 @@ lame_decoder(lame_global_flags * gfp, FILE * outf, int skip, char *inPath,
             if (enc_delay>-1) skip = enc_delay + 528+1;
             else skip=lame_get_encoder_delay(gfp)+528+1;
         }else{
-            // user specified a value of skip. just add for decoder
+            /* user specified a value of skip. just add for decoder */
             skip += 528+1; /* mp3 decoder has a 528 sample delay, plus user supplied "skip" */
         }
 
@@ -208,10 +213,6 @@ lame_decoder(lame_global_flags * gfp, FILE * outf, int skip, char *inPath,
         if (silent < 10) fprintf(stderr, "MPEG-%u%s Layer %s", 2 - lame_get_version(gfp),
                 lame_get_out_samplerate( gfp ) < 16000 ? ".5" : "", "I");
         break;
-    case sf_ogg:
-        if (silent < 10) fprintf(stderr, "Ogg Vorbis");
-        skip = 0;       /* other formats have no delay *//* is += 0 not better ??? */
-        break;
     case sf_raw:
         if (silent < 10) fprintf(stderr, "raw PCM data");
         mp3input_data.nsamp = lame_get_num_samples( gfp );
@@ -239,13 +240,16 @@ lame_decoder(lame_global_flags * gfp, FILE * outf, int skip, char *inPath,
         break;
     }
 
-    if (silent < 10) fprintf(stderr, ")\noutput: %s%s(16 bit, Microsoft WAVE)\n",
-            strcmp(outPath, "-") ? outPath : "<stdout>",
-            strlen(outPath) > 45 ? "\n\t" : "  ");
+    if (silent < 10) {
+	fprintf(stderr, ")\noutput: %s%s(16 bit, Microsoft WAVE)\n",
+		strcmp(outPath, "-") ? outPath : "<stdout>",
+		strlen(outPath) > 45 ? "\n\t" : "  ");
 
-    if (skip > 0)
-        if (silent < 10) fprintf(stderr, "skipping initial %i samples (encoder+decoder delay)\n",
-                skip);
+	if (skip > 0)
+	    fprintf(stderr,
+		    "skipping initial %i samples (encoder+decoder delay)\n",
+		    skip);
+    }
 
     if ( 0 == disable_wav_header )
         WriteWaveHeader(outf, 0x7FFFFFFF, lame_get_in_samplerate( gfp ),
@@ -300,7 +304,7 @@ lame_decoder(lame_global_flags * gfp, FILE * outf, int skip, char *inPath,
 
     if ( 0 == disable_wav_header )
         if (!fseek(outf, 0l, SEEK_SET)) /* if outf is seekable, rewind and adjust length */
-            WriteWaveHeader(outf, wavsize, lame_get_in_samplerate( gfp ),
+            WriteWaveHeader(outf, (int)wavsize, lame_get_in_samplerate( gfp ),
                             tmp_num_channels, 16);
     fclose(outf);
 
@@ -347,10 +351,7 @@ lame_encoder(lame_global_flags * gf, FILE * outf, int nogap, char *inPath,
         fprintf(stderr,
                 "Encoding as %g kHz ", 1.e-3 * lame_get_out_samplerate(gf));
 
-        if (lame_get_ogg(gf)) {
-            fprintf(stderr, "VBR Ogg Vorbis\n");
-        }
-        else {
+	{
             const char *appendix = "";
 
             switch (lame_get_VBR(gf)) {
@@ -433,14 +434,10 @@ lame_encoder(lame_global_flags * gf, FILE * outf, int nogap, char *inPath,
 
     } while (iread);
 
-    if (nogap) {
+    if (nogap) 
         imp3 = lame_encode_flush_nogap(gf, mp3buffer, sizeof(mp3buffer)); /* may return one more mp3 frame */
-        /* reinitialize bitstream for next encoding.  this is normally done
-         * by lame_init_params(), but we cannot call that routine twice */
-        lame_init_bitstream(gf);
-    } else {
+    else
         imp3 = lame_encode_flush(gf, mp3buffer, sizeof(mp3buffer)); /* may return one more mp3 frame */
-    }
 
     if (imp3 < 0) {
         if (imp3 == -1)
@@ -489,7 +486,7 @@ brhist_init_package(lame_global_flags * gf)
         }
     }
     else {
-        brhist_init(gf, 128, 128); // Dirty hack
+        brhist_init(gf, 128, 128); /* Dirty hack */
     }
 #endif
 }
@@ -665,10 +662,6 @@ main(int argc, char **argv)
          * specify the output file name, only an optional output directory. */
         parse_nogap_filenames(nogapout,nogap_inPath[0],outPath,nogapdir);
         outf = init_files(gf, nogap_inPath[0], outPath);
-        if (lame_get_bWriteVbrTag(gf)) {
-            fprintf(stderr,"Note: Disabling VBR Xing/Info tag since it interferes with --nogap\n");
-            lame_set_bWriteVbrTag( gf, 0 );
-        }
     }
     else {
         outf = init_files(gf, inPath, outPath);
@@ -694,14 +687,6 @@ main(int argc, char **argv)
     }
 
 
-#ifdef HAVE_VORBIS_ENCODER
-    if (lame_get_ogg(gf)) {
-        lame_encode_ogg_init(gf);
-        lame_set_VBR(gf, vbr_off); /* ignore lame's various VBR modes */
-    }
-#endif
-
-
     if (lame_get_decode_only(gf)) {
         /* decode an mp3 file to a .wav */
         if (mp3_delay_set)
@@ -734,6 +719,11 @@ main(int argc, char **argv)
                 
                 fclose(outf); /* close the output file */
                 close_infile(); /* close the input file */
+
+                /* reinitialize bitstream for next encoding.  this is normally done
+                 * by lame_init_params(), but we cannot call that routine twice */
+                if (use_flush_nogap) 
+                    lame_init_bitstream(gf);
             }
             lame_close(gf);
 
@@ -756,3 +746,4 @@ main(int argc, char **argv)
     }
     return ret;
 }
+
