@@ -19,28 +19,19 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: VbrTag.c,v 1.58 2002/10/08 19:30:00 bouvigne Exp $ */
+/* $Id: VbrTag.c,v 1.74 2004/01/06 13:53:23 bouvigne Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
 #include "machine.h"
-#if defined(__riscos__) && defined(FPA10)
-#include	"ymath.h"
-#else 
-#include	<math.h>
-#endif
-
-
 #include "bitstream.h"
 #include "lame.h"
 #include "VbrTag.h"
 #include "version.h"
 
 #include	<assert.h>
-#include 	<stdlib.h>
-#include 	<string.h>
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
@@ -57,15 +48,15 @@
 #endif
 
 /*
-//    4 bytes for Header Tag
-//    4 bytes for Header Flags
-//  100 bytes for entry (NUMTOCENTRIES)
-//    4 bytes for FRAME SIZE
-//    4 bytes for STREAM_SIZE
-//    4 bytes for VBR SCALE. a VBR quality indicator: 0=best 100=worst
-//   20 bytes for LAME tag.  for example, "LAME3.12 (beta 6)"
-// ___________
-//  140 bytes
+ *    4 bytes for Header Tag 
+ *    4 bytes for Header Flags 
+ *  100 bytes for entry (NUMTOCENTRIES) 
+ *    4 bytes for FRAME SIZE 
+ *    4 bytes for STREAM_SIZE 
+ *    4 bytes for VBR SCALE. a VBR quality indicator: 0=best 100=worst 
+ *   20 bytes for LAME tag.  for example, "LAME3.12 (beta 6)" 
+ * ___________ 
+ *  140 bytes 
 */
 #define VBRHEADERSIZE (NUMTOCENTRIES+4+4+4+4+4)
 
@@ -179,7 +170,6 @@ static void Xing_seek_table(VBR_seek_info_t * v, unsigned char *t)
     }
 }
 
-#if 0
 void print_seeking(unsigned char *t)
 {
     int i;
@@ -190,7 +180,6 @@ void print_seeking(unsigned char *t)
     }
     printf("\n");
 }
-#endif
 
 
 
@@ -206,22 +195,7 @@ void AddVbrFrame(lame_global_flags *gfp)
     lame_internal_flags *gfc = gfp->internal_flags;
 
     int kbps = bitrate_table[gfp->version][gfc->bitrate_index];
-    
-    if (gfc->VBR_seek_table.bag == NULL) {
-        gfc->VBR_seek_table.sum  = 0;
-        gfc->VBR_seek_table.seen = 0;
-        gfc->VBR_seek_table.want = 1;
-        gfc->VBR_seek_table.pos  = 0;
-        gfc->VBR_seek_table.bag  = malloc (400*sizeof(int));
-        if (gfc->VBR_seek_table.bag != NULL) {
-            gfc->VBR_seek_table.size = 400;
-        }
-        else {
-            gfc->VBR_seek_table.size = 0;
-            ERRORF (gfc,"Error: can't allocate VbrFrames buffer\n");
-            return;
-        }   
-    }
+    assert(gfc->VBR_seek_table.bag);
     addVbr(&gfc->VBR_seek_table, kbps);
     gfp->nVbrNumFrames++;
 }
@@ -316,8 +290,8 @@ int GetVbrTag(VBRTAGDATA *pTagData,  unsigned char *buf)
             pTagData->samprate = samplerate_table[2][h_sr_index];
         else
             pTagData->samprate = samplerate_table[h_id][h_sr_index];
-        //	if( h_id == 0 )
-        //		pTagData->samprate >>= 1;
+        /*	if( h_id == 0 ) */
+        /*		pTagData->samprate >>= 1; */
 
 
 
@@ -381,8 +355,8 @@ int GetVbrTag(VBRTAGDATA *pTagData,  unsigned char *buf)
         enc_delay += buf[1] >> 4;
         enc_padding= (buf[1] & 0x0F)<<8;
         enc_padding += buf[2];
-        // check for reasonable values (this may be an old Xing header,
-        // not a INFO tag)
+        /* check for reasonable values (this may be an old Xing header, */
+        /* not a INFO tag) */
         if (enc_delay<0 || enc_delay > 3000) enc_delay=-1;
         if (enc_padding<0 || enc_padding > 3000) enc_padding=-1;
 
@@ -423,68 +397,86 @@ int GetVbrTag(VBRTAGDATA *pTagData,  unsigned char *buf)
 int InitVbrTag(lame_global_flags *gfp)
 {
 	int nMode,SampIndex;
+	int i,kbps_header,tot;
 	lame_internal_flags *gfc = gfp->internal_flags;
-#define MAXFRAMESIZE 2880 // or 0xB40, the max freeformat 640 32kHz framesize
-	//	uint8_t pbtStreamBuffer[MAXFRAMESIZE];
+#define MAXFRAMESIZE 2880 /* or 0xB40, the max freeformat 640 32kHz framesize */
+	/*	uint8_t pbtStreamBuffer[MAXFRAMESIZE]; */
 	nMode = gfp->mode;
 	SampIndex = gfc->samplerate_index;
 
 
-	/* Clear Frame position array variables */
-	//gfp->pVbrFrames=NULL;
+
 	gfp->nVbrNumFrames=0;
-	gfp->nVbrFrameBufferSize=0;
+	/*gfp->nVbrFrameBufferSize=0;*/
 
 
 	/* Clear stream buffer */
-	//	memset(pbtStreamBuffer,0x00,sizeof(pbtStreamBuffer));
+	/*	memset(pbtStreamBuffer,0x00,sizeof(pbtStreamBuffer)); */
 
 
 
 	/*
-	// Xing VBR pretends to be a 48kbs layer III frame.  (at 44.1kHz).
-        // (at 48kHz they use 56kbs since 48kbs frame not big enough for
-        // table of contents)
-	// let's always embed Xing header inside a 64kbs layer III frame.
-	// this gives us enough room for a LAME version string too.
-	// size determined by sampling frequency (MPEG1)
-	// 32kHz:    216 bytes@48kbs    288bytes@ 64kbs
-	// 44.1kHz:  156 bytes          208bytes@64kbs     (+1 if padding = 1)
-	// 48kHz:    144 bytes          192
-	//
-	// MPEG 2 values are the same since the framesize and samplerate
-        // are each reduced by a factor of 2.
+	 * Xing VBR pretends to be a 48kbs layer III frame.  (at 44.1kHz). 
+         * (at 48kHz they use 56kbs since 48kbs frame not big enough for 
+         * table of contents) 
+	 * let's always embed Xing header inside a 64kbs layer III frame. 
+	 * this gives us enough room for a LAME version string too. 
+	 * size determined by sampling frequency (MPEG1) 
+	 * 32kHz:    216 bytes@48kbs    288bytes@ 64kbs 
+	 * 44.1kHz:  156 bytes          208bytes@64kbs     (+1 if padding = 1) 
+	 * 48kHz:    144 bytes          192 
+	 * 
+	 * MPEG 2 values are the same since the framesize and samplerate 
+         * are each reduced by a factor of 2. 
 	*/
-	{
-	int i,bitrate,tot;
+
+
 	if (1==gfp->version) {
-	  bitrate = XING_BITRATE1;
+	  kbps_header = XING_BITRATE1;
 	} else {
 	  if (gfp->out_samplerate < 16000 )
-	    bitrate = XING_BITRATE25;
+	    kbps_header = XING_BITRATE25;
 	  else
-	    bitrate = XING_BITRATE2;
+	    kbps_header = XING_BITRATE2;
 	}
 
 	if (gfp->VBR==vbr_off)
-			bitrate = gfp->brate;
+			kbps_header = gfp->brate;
 	
 	gfp->TotalFrameSize= 
-  	  ((gfp->version+1)*72000*bitrate) / gfp->out_samplerate;
+  	  ((gfp->version+1)*72000*kbps_header) / gfp->out_samplerate;
 
 	tot = (gfc->sideinfo_len+LAMEHEADERSIZE);
 
 	if (gfp->TotalFrameSize < tot || 
             gfp->TotalFrameSize > MAXFRAMESIZE ) {
-            // disable tag, it wont fit
+            /* disable tag, it wont fit */
             gfp->bWriteVbrTag = 0;
             return 0;
         }
 
+        /* write dummy VBR tag of all 0's into bitstream */ 
 	for (i=0; i<gfp->TotalFrameSize; ++i)
 	  add_dummy_byte(gfp,0);
-	}
 
+
+        gfc->VBR_seek_table.sum  = 0;
+        
+        gfc->VBR_seek_table.seen = 0;
+        gfc->VBR_seek_table.want = 1;
+        gfc->VBR_seek_table.pos  = 0;
+        
+        if (gfc->VBR_seek_table.bag == NULL) {
+            gfc->VBR_seek_table.bag  = malloc (400*sizeof(int));
+            if (gfc->VBR_seek_table.bag != NULL) {
+                gfc->VBR_seek_table.size = 400;
+            }
+            else {
+                gfc->VBR_seek_table.size = 0;
+                ERRORF (gfc,"Error: can't allocate VbrFrames buffer\n");
+                return -1;
+            }   
+        }
 	/* Success */
 	return 0;
 }
@@ -511,13 +503,59 @@ void UpdateMusicCRC(uint16_t *crc,unsigned char *buffer, int size){
 
 void ReportLameTagProgress(lame_global_flags *gfp,int nStart)
 {
-	if (!gfp->bWriteVbrTag)
-		return;
+	if (nStart) {
+		if (gfp->bWriteVbrTag)
+		  MSGF( gfp->internal_flags, "Writing LAME Tag...");
+	}
+	else {
+		lame_internal_flags * gfc = gfp->internal_flags;
 
-	if (nStart)
-		MSGF( gfp->internal_flags, "Writing LAME Tag...");
-	else
-		MSGF( gfp->internal_flags, "done\n");
+		if (gfp->bWriteVbrTag)
+		  MSGF( gfc, "done\n");
+
+		if( gfc->findReplayGain ) {
+		  MSGF( gfc, "ReplayGain: %s%.1fdB\n", gfc->RadioGain > 0 ? "+" : "", ((FLOAT)gfc->RadioGain) / 10.0 );
+		  if( gfc->RadioGain > 0x1FE || gfc->RadioGain < -0x1FE ) {
+		    MSGF( gfc, "WARNING: ReplayGain  exceeds  -51dB to +51dB  range.  Such a result is too\n" );
+		    MSGF( gfc, "         high to be stored in the header.\n" );
+		  }
+		}
+                                                                                         
+#ifdef DECODE_ON_THE_FLY
+		if( gfp->findPeakSample ) {
+		  FLOAT GainChange = 20.0 * log10( ((FLOAT)gfc->PeakSample) / 32767.0 );
+
+		  if (GainChange > 0.0) { /* clipping occurs */
+		    MSGF( gfc, "\nWARNING: clipping occurs at the current gain. Set your decoder to decrease\n");
+		    MSGF( gfc,   "         the  gain  by  at least %.1fdB or encode again ", ceil(GainChange * 10.0) / 10.0 );  /* round up */
+                                                                            
+		    /* advice the user on the scale factor */
+		    if (gfp->scale == 1.0 || gfp->scale == 0.0) {
+		      FLOAT scale = floor( (32767.0 / ((FLOAT)gfc->PeakSample)) * 100.0 ) / 100.0;  /* round down */
+		      MSGF( gfc, "using  --scale %.2f\n", scale);
+		      MSGF( gfc, "         or less (the value under --scale is approximate).\n" );
+		    }
+		    else {                      
+                      /* the user specified his own scale factor. We could suggest 
+                       * the scale factor of (32767.0/gfc->PeakSample)*(gfp->scale)
+                       * but it's usually very inaccurate. So we'd rather advice him to 
+                       * disable scaling first and see our suggestion on the scale factor then. */
+		      MSGF( gfc, "using --scale <arg>\n");
+		      MSGF( gfc, "         (For   a   suggestion  on  the  optimum  value  of  <arg>  encode\n" );
+		      MSGF( gfc, "         with  --scale 1  first)\n" );
+		    }
+		    
+		  }
+		  else { /* no clipping */
+		    if (GainChange > -0.1)
+		      MSGF( gfc, "\nThe MP3 file does not clip and is less than 0.1dB away from full scale.\n" );
+		    else                                                                                        
+		      MSGF( gfc, "\nThe MP3 file does not clip and is at least %.1fdB away from full scale.\n", floor((-GainChange) * 10.0) / 10.0);  /* round down */
+		  }                                                                                             
+		}
+#endif
+
+	}
 
 }
 
@@ -536,17 +574,17 @@ void ReportLameTagProgress(lame_global_flags *gfp,int nStart)
 int PutLameVBR(lame_global_flags *gfp, FILE *fpStream, uint8_t *pbtStreamBuffer, uint32_t id3v2size,  uint16_t crc)
 {
     lame_internal_flags *gfc = gfp->internal_flags;
-//	FLOAT fVersion = LAME_MAJOR_VERSION + 0.01 * LAME_MINOR_VERSION;
+/*	FLOAT fVersion = LAME_MAJOR_VERSION + 0.01 * LAME_MINOR_VERSION; */
 
 	int nBytesWritten = 0;
-	int nFilesize	  = 0;		//size of fpStream. Will be equal to size after process finishes.
+	int nFilesize	  = 0;		/*size of fpStream. Will be equal to size after process finishes. */
 	int i;
 
-    int enc_delay=lame_get_encoder_delay(gfp);       // encoder delay
-    int enc_padding=lame_get_encoder_padding(gfp);   // encoder padding 
+    int enc_delay=lame_get_encoder_delay(gfp);       /* encoder delay */
+    int enc_padding=lame_get_encoder_padding(gfp);   /* encoder padding  */
 
-	//recall:	gfp->VBR_q is for example set by the switch -V 
-	//			gfp->quality by -q, -h, -f, etc
+	/*recall:	gfp->VBR_q is for example set by the switch -V  */
+	/*			gfp->quality by -q, -h, -f, etc */
 	
 	int nQuality		= (100 - 10 * gfp->VBR_q - gfp->quality);
 	
@@ -555,29 +593,29 @@ int PutLameVBR(lame_global_flags *gfp, FILE *fpStream, uint8_t *pbtStreamBuffer,
 	uint8_t nVBR;
 	uint8_t nRevision = 0x00;
 	uint8_t nRevMethod;
-	uint8_t vbr_type_translator[] = {1,5,3,2,4,0,3};		//numbering different in vbr_mode vs. Lame tag
+	uint8_t vbr_type_translator[] = {1,5,3,2,4,0,3};		/*numbering different in vbr_mode vs. Lame tag */
 
 	uint8_t nLowpass		= ( ((gfp->lowpassfreq / 100.0)+.5) > 255 ? 255 : (gfp->lowpassfreq / 100.0)+.5 );
 
-	ieee754_float32_t fPeakSignalAmplitude	= 0;				//TODO...
-	uint16_t nRadioReplayGain		= 0;				//TODO...
-	uint16_t nAudioPhileReplayGain  = 0;				//TODO...
-
-
-
+#ifdef DECODE_ON_THE_FLY
+    uint32_t nPeakSignalAmplitude = abs((int)((((FLOAT8)gfc->PeakSample) / 32767.0 ) * pow(2,23) +.5));
+#else
+    uint32_t nPeakSignalAmplitude = 0;
+#endif
+  	uint16_t nRadioReplayGain		= 0; 
+	uint16_t nAudiophileReplayGain		= 0;
 
 	uint8_t nNoiseShaping			= gfp->internal_flags->noise_shaping;
 	uint8_t nStereoMode				= 0;
 	int		bNonOptimal				= 0;
 	uint8_t nSourceFreq				= 0;
 	uint8_t nMisc					= 0;
-	uint16_t nPresetValue			= 0;
 	uint32_t nMusicLength			= 0;
 	int		bId3v1Present			= ((gfp->internal_flags->tag_spec.flags & CHANGED_FLAG)
 		&& !(gfp->internal_flags->tag_spec.flags & V2_ONLY_FLAG));
 	uint16_t nMusicCRC				= 0;
 
-	//psy model type: Gpsycho or NsPsytune
+	/*psy model type: Gpsycho or NsPsytune */
 	unsigned char    bExpNPsyTune	= gfp->exp_nspsytune & 1;
 	unsigned char	 bSafeJoint		= (gfp->exp_nspsytune & 2)!=0;
 
@@ -588,22 +626,56 @@ int PutLameVBR(lame_global_flags *gfp, FILE *fpStream, uint8_t *pbtStreamBuffer,
 	int		 nNoGapCurr		= gfp->internal_flags->nogap_current;
 
 
-	uint8_t  nAthType		= gfp->ATHtype;	//4 bits.
+	uint8_t  nAthType		= gfp->ATHtype;	/*4 bits. */
 	
 	uint8_t  nFlags			= 0;
 
-	// if ABR, {store bitrate <=255} else { store "-b"}
-	int nABRBitrate	= (gfp->VBR==vbr_abr)?gfp->VBR_mean_bitrate_kbps:gfp->brate;
+	/* if ABR, {store bitrate <=255} else { store "-b"} */
+	int nABRBitrate;
+    switch (gfp->VBR) {
+        case vbr_abr:{
+            nABRBitrate = gfp->VBR_mean_bitrate_kbps;
+            break;
+        }
+        case vbr_off:{
+            nABRBitrate = gfp->brate;
+            break;
+        }
+        default:{ /*vbr modes*/
+            nABRBitrate = gfp->VBR_min_bitrate_kbps;
+        }
+    }
+        
 
-	//revision and vbr method
+	/*revision and vbr method */
 	if (gfp->VBR>=0 && gfp->VBR < sizeof(vbr_type_translator))
 		nVBR = vbr_type_translator[gfp->VBR];
 	else
-		nVBR = 0x00;		//unknown.
+		nVBR = 0x00;		/*unknown. */
 
 	nRevMethod = 0x10 * nRevision + nVBR; 
+
+
+	/* ReplayGain */
+	if (gfc->findReplayGain) { 
+	  if (gfc->RadioGain > 0x1FE)
+	    gfc->RadioGain = 0x1FE;
+	  if (gfc->RadioGain < -0x1FE)
+	    gfc->RadioGain = -0x1FE;
+
+	  nRadioReplayGain = 0x2000; /* set name code */
+	  nRadioReplayGain |= 0xC00; /* set originator code to `determined automatically' */
+
+	  if (gfc->RadioGain >= 0) 
+	    nRadioReplayGain |= gfc->RadioGain; /* set gain adjustment */
+	  else {
+            nRadioReplayGain |= 0x200; /* set the sign bit */
+	    nRadioReplayGain |= -gfc->RadioGain; /* set gain adjustment */
+	  }
+	}
 	
-	//nogap
+
+	/*nogap */
 	if (nNoGapCount != -1)
 	{
 		if (nNoGapCurr > 0)
@@ -613,7 +685,7 @@ int PutLameVBR(lame_global_flags *gfp, FILE *fpStream, uint8_t *pbtStreamBuffer,
 			bNoGapMore = 1;
 	}
 
-	//flags
+	/*flags */
 
 	nFlags	= nAthType	+ (bExpNPsyTune		<< 4)
 						+ (bSafeJoint		<< 5)
@@ -650,9 +722,6 @@ int PutLameVBR(lame_global_flags *gfp, FILE *fpStream, uint8_t *pbtStreamBuffer,
 		break;
 	}
 
-	if (gfp->mode_automs)
-		nStereoMode = 5;
-
 	/*Intensity stereo : nStereoMode = 6. IS is not implemented */
 
 	if (gfp->in_samplerate <= 32000)
@@ -662,14 +731,14 @@ int PutLameVBR(lame_global_flags *gfp, FILE *fpStream, uint8_t *pbtStreamBuffer,
 	else if (gfp->in_samplerate > 48000)
 		nSourceFreq = 0x03;
 	else
-		nSourceFreq = 0x01;  //default is 44100Hz.
+		nSourceFreq = 0x01;  /*default is 44100Hz. */
 
 
-	//Check if the user overrided the default LAME behaviour with some nasty options
+	/*Check if the user overrided the default LAME behaviour with some nasty options */
 
 	if (gfp->short_blocks == short_block_forced 			||
 		gfp->short_blocks == short_block_dispensed 		||
-		((gfp->lowpassfreq == -1) && (gfp->highpassfreq == -1))	|| // "-k"
+		((gfp->lowpassfreq == -1) && (gfp->highpassfreq == -1))	|| /* "-k" */
 		(gfp->scale_left != gfp->scale_right)			||
 		gfp->disable_reservoir		||
 		gfp->noATH			||
@@ -685,15 +754,14 @@ int PutLameVBR(lame_global_flags *gfp, FILE *fpStream, uint8_t *pbtStreamBuffer,
 
 
 	
-	//get filesize
+	/*get filesize */
 	fseek(fpStream, 0, SEEK_END);
 	nFilesize = ftell(fpStream);
 
-    nPresetValue = gfp->preset;
 	
-	nMusicLength = nFilesize - id3v2size;		//omit current frame
+	nMusicLength = nFilesize - id3v2size;		/*omit current frame */
 	if (bId3v1Present)
-		nMusicLength-=128;                     //id3v1 present.
+		nMusicLength-=128;                     /*id3v1 present. */
         nMusicCRC = gfc->nMusicCRC;
 
 
@@ -710,13 +778,13 @@ int PutLameVBR(lame_global_flags *gfp, FILE *fpStream, uint8_t *pbtStreamBuffer,
 	pbtStreamBuffer[nBytesWritten] = nLowpass;
 	nBytesWritten++;
 
-	memmove(&pbtStreamBuffer[nBytesWritten], &fPeakSignalAmplitude, 4);
+	CreateI4(&pbtStreamBuffer[nBytesWritten], nPeakSignalAmplitude);
 	nBytesWritten+=4;
 
 	CreateI2(&pbtStreamBuffer[nBytesWritten],nRadioReplayGain);
 	nBytesWritten+=2;
 
-	CreateI2(&pbtStreamBuffer[nBytesWritten],nAudioPhileReplayGain);
+	CreateI2(&pbtStreamBuffer[nBytesWritten],nAudiophileReplayGain);
 	nBytesWritten+=2;
 
 	pbtStreamBuffer[nBytesWritten] = nFlags;
@@ -728,7 +796,7 @@ int PutLameVBR(lame_global_flags *gfp, FILE *fpStream, uint8_t *pbtStreamBuffer,
 		pbtStreamBuffer[nBytesWritten] = nABRBitrate;
 	nBytesWritten++;
 
-        pbtStreamBuffer[nBytesWritten   ] = enc_delay >> 4; // works for win32, does it for unix?
+        pbtStreamBuffer[nBytesWritten   ] = enc_delay >> 4; /* works for win32, does it for unix? */
         pbtStreamBuffer[nBytesWritten +1] = (enc_delay << 4) + (enc_padding >> 8);
         pbtStreamBuffer[nBytesWritten +2] = enc_padding;
 
@@ -738,10 +806,9 @@ int PutLameVBR(lame_global_flags *gfp, FILE *fpStream, uint8_t *pbtStreamBuffer,
 	nBytesWritten++;
 
 
-	memset(pbtStreamBuffer+nBytesWritten,0, 1);		//unused in rev1
-	nBytesWritten+=1;
+	pbtStreamBuffer[nBytesWritten++] = 0;	/*unused in rev0 */
 
-	CreateI2(&pbtStreamBuffer[nBytesWritten], nPresetValue);
+	CreateI2(&pbtStreamBuffer[nBytesWritten], gfp->preset);
 	nBytesWritten+=2;
 
 	CreateI4(&pbtStreamBuffer[nBytesWritten], nMusicLength);
@@ -851,8 +918,11 @@ int PutVbrTag(lame_global_flags *gfp,FILE *fpStream,int nVbrScale)
 		
 		if (gfp->VBR==vbr_off)
 			bitrate = gfp->brate;
-
-		bbyte = 16*BitrateIndex(bitrate,gfp->version,gfp->out_samplerate);
+        
+        if (gfp->free_format)
+            bbyte = 0x00;
+        else
+		    bbyte = 16*BitrateIndex(bitrate,gfp->version,gfp->out_samplerate);
 	}
 
 	/* Use as much of the info from the real frames in the
@@ -873,8 +943,14 @@ int PutVbrTag(lame_global_flags *gfp,FILE *fpStream,int nVbrScale)
 	/* Clear all TOC entries */
 	memset(btToc,0,sizeof(btToc));
 
-        Xing_seek_table (&gfc->VBR_seek_table, btToc);
-        /* print_seeking (btToc); */
+    if (gfp->free_format) {
+        int i;
+        for (i = 1; i < NUMTOCENTRIES; ++i)
+            btToc[i] = 255*i/100;
+    } else {
+        Xing_seek_table(&gfc->VBR_seek_table, btToc);
+    }
+    /*print_seeking (btToc);*/
 
 	/* Start writing the tag after the zero frame */
 	nStreamIndex=gfc->sideinfo_len;
@@ -926,7 +1002,7 @@ int PutVbrTag(lame_global_flags *gfp,FILE *fpStream,int nVbrScale)
 
 
 
-	//work out CRC so far: initially crc = 0
+	/*work out CRC so far: initially crc = 0 */
 	for (i = 0;i< nStreamIndex ;i++)
 		crc = CRC_update_lookup(pbtStreamBuffer[i], crc);
 
@@ -948,10 +1024,9 @@ int PutVbrTag(lame_global_flags *gfp,FILE *fpStream,int nVbrScale)
 	{
 		return -1;
 	}
-	/* Save to delete the frame buffer */
-	//free(gfp->pVbrFrames);  see HACKING for instructions on how
-	//gfp->pVbrFrames=NULL;   memory in 'gfp' is allocated/free'd
+
 
 	return 0;       /* success */
 }
 
+
