@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: bitstream.c,v 1.57 2002/05/07 20:15:13 robert Exp $
+ * $Id: bitstream.c,v 1.60 2003/10/22 07:33:49 takehiro Exp $
  */
 
 
@@ -35,15 +35,14 @@
 #include "quantize_pvt.h"
 #include "version.h"
 #include "VbrTag.h"
+#ifdef DECODE_ON_THE_FLY
+#include "gain_analysis.h"
+#endif
 
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
 #endif
 
-/* This is the scfsi_band table from 2.4.2.7 of the IS */
-const int scfsi_band[5] = { 0, 6, 11, 16, 21 };
-
-
 /* unsigned int is at least this large:  */
 /* we work with ints, so when doing bit manipulation, we limit
  * ourselves to MAX_LENGTH-2 just to be on the safe side */
@@ -59,6 +58,30 @@ static int hoge, hogege;
 
 
 
+/***********************************************************************
+ * compute bitsperframe and mean_bits for a layer III frame 
+ **********************************************************************/
+int getframebits(const lame_global_flags * gfp)
+{
+    lame_internal_flags *gfc=gfp->internal_flags;
+    int  bit_rate;
+
+    /* get bitrate in kbps [?] */
+    if (gfc->bitrate_index) 
+	bit_rate = bitrate_table[gfp->version][gfc->bitrate_index];
+    else
+	bit_rate = gfp->brate;
+    assert ( bit_rate <= 550 );
+
+    /* main encoding routine toggles padding on and off */
+    /* one Layer3 Slot consists of 8 bits */
+    return 8 * ((gfp->version+1)*72000*bit_rate / gfp->out_samplerate
+		+ gfc->padding);
+}
+
+
+
+
 void putheader_bits(lame_internal_flags *gfc,int w_ptr)
 {
     Bit_stream_struc *bs;
@@ -152,8 +175,9 @@ putbits_noheaders(lame_internal_flags *gfc, int val, int j)
 */
 
 inline static void
-drain_into_ancillary(lame_internal_flags *gfc, int remainingBits)
+drain_into_ancillary(lame_global_flags *gfp, int remainingBits)
 {
+    lame_internal_flags *gfc=gfp->internal_flags;
     int i;
     assert(remainingBits >= 0);
 
@@ -185,7 +209,7 @@ drain_into_ancillary(lame_internal_flags *gfc, int remainingBits)
 
     for (; remainingBits >= 1; remainingBits -= 1 ) {
         putbits2(gfc, gfc->ancillary_flag, 1 );
-        gfc->ancillary_flag ^= 1;
+	gfc->ancillary_flag ^= !gfp->disable_reservoir;
     }
 
     assert (remainingBits == 0);
@@ -292,13 +316,13 @@ encodeSideInfo2(lame_global_flags *gfp,int bitsPerFrame)
 	for (gr = 0; gr < 2; gr++) {
 	    for (ch = 0; ch < gfc->channels_out; ch++) {
 		gr_info *gi = &l3_side->tt[gr][ch];
-		writeheader(gfc,gi->part2_3_length,       12);
+		writeheader(gfc,gi->part2_3_length+gi->part2_length, 12);
 		writeheader(gfc,gi->big_values / 2,        9);
 		writeheader(gfc,gi->global_gain,           8);
 		writeheader(gfc,gi->scalefac_compress,     4);
 
 		if (gi->block_type != NORM_TYPE) {
-		    writeheader(gfc, 1, 1); // window_switching_flag
+		    writeheader(gfc, 1, 1); /* window_switching_flag */
 		    writeheader(gfc,gi->block_type,       2);
 		    writeheader(gfc,gi->mixed_block_flag, 1);
 
@@ -313,7 +337,7 @@ encodeSideInfo2(lame_global_flags *gfp,int bitsPerFrame)
 		    writeheader(gfc,gi->subblock_gain[1], 3);
 		    writeheader(gfc,gi->subblock_gain[2], 3);
 		} else {
-		    writeheader(gfc, 0, 1); // window_switching_flag
+		    writeheader(gfc, 0, 1); /* window_switching_flag */
 		    if (gi->table_select[0] == 14)
 			gi->table_select[0] = 16;
 		    writeheader(gfc,gi->table_select[0], 5);
@@ -343,13 +367,13 @@ encodeSideInfo2(lame_global_flags *gfp,int bitsPerFrame)
 	gr = 0;
 	for (ch = 0; ch < gfc->channels_out; ch++) {
 	    gr_info *gi = &l3_side->tt[gr][ch];
-	    writeheader(gfc,gi->part2_3_length,       12);
+	    writeheader(gfc,gi->part2_3_length+gi->part2_length, 12);
 	    writeheader(gfc,gi->big_values / 2,        9);
 	    writeheader(gfc,gi->global_gain,           8);
 	    writeheader(gfc,gi->scalefac_compress,     9);
 
 	    if (gi->block_type != NORM_TYPE) {
-		writeheader(gfc, 1, 1); // window_switching_flag
+		writeheader(gfc, 1, 1); /* window_switching_flag */
 		writeheader(gfc,gi->block_type,       2);
 		writeheader(gfc,gi->mixed_block_flag, 1);
 
@@ -364,7 +388,7 @@ encodeSideInfo2(lame_global_flags *gfp,int bitsPerFrame)
 		writeheader(gfc,gi->subblock_gain[1], 3);
 		writeheader(gfc,gi->subblock_gain[2], 3);
 	    } else {
-		writeheader(gfc, 0, 1); // window_switching_flag
+		writeheader(gfc, 0, 1); /* window_switching_flag */
 		if (gi->table_select[0] == 14)
 		    gi->table_select[0] = 16;
 		writeheader(gfc,gi->table_select[0], 5);
@@ -467,7 +491,8 @@ huffman_coder_count1(lame_internal_flags *gfc, gr_info *gi)
 	bits += h->hlen[p];
     }
 #ifdef DEBUG
-    DEBUGF(gfc,"%ld %d %d %d\n",gfc->bs.totbit -gegebo, gi->count1bits, gi->big_values, gi->count1);
+    DEBUGF(gfc,"count1: real: %ld counted:%d (bigv %d count1len %d)\n",
+	   gfc->bs.totbit -gegebo, gi->count1bits, gi->big_values, gi->count1);
 #endif
     return bits;
 }
@@ -477,91 +502,76 @@ huffman_coder_count1(lame_internal_flags *gfc, gr_info *gi)
 /*
   Implements the pseudocode of page 98 of the IS
   */
-
 inline static int
-HuffmanCode( lame_internal_flags* const gfc, const int table_select,
-	     int index, gr_info *gi)
+Huffmancode( lame_internal_flags* const gfc, const int tableindex,
+	     int start, int end, gr_info *gi)
 {
-    const struct huffcodetab* h = ht + table_select;
-    int  cbits   = 0;
-    int  xbits   = 0;
-    int  linbits = h->xlen;
-    int  xlen    = h->xlen;
-    int  ext = 0;
-    int x1 = gi->l3_enc[index];
-    int x2 = gi->l3_enc[index+1];
+    const struct huffcodetab* h = &ht[tableindex];
+    int index, bits = 0;
 
-    assert ( table_select > 0 );
+    assert(tableindex < 32u);
+    if (!tableindex)
+	return bits;
 
-    if (x1 != 0) {
-	if (gi->xr[index] < 0)
-	    ext++;
+    for (index = start; index < end; index += 2) {
+	int cbits   = 0;
+	int xbits   = 0;
+	int linbits = h->xlen;
+	int xlen    = h->xlen;
+	int ext = 0;
+	int x1 = gi->l3_enc[index];
+	int x2 = gi->l3_enc[index+1];
 
-	cbits--;
-    }
-
-    if (table_select > 15) {
-	/* use ESC-words */
-	if (x1 > 14) {
-	    int linbits_x1 = x1 - 15;
-	    assert ( linbits_x1 <= h->linmax );
-	    ext   |= linbits_x1 << 1;
-	    xbits  = linbits;
-	    x1     = 15;
+	if (x1 != 0) {
+	    if (gi->xr[index] < 0)
+		ext++;
+	    cbits--;
 	}
 
-	if (x2 > 14) {
-	    int linbits_x2 = x2 - 15;
-	    assert ( linbits_x2 <= h->linmax );
-	    ext  <<= linbits;
-	    ext   |= linbits_x2;
-	    xbits += linbits;
-	    x2     = 15;
+	if (tableindex > 15) {
+	    /* use ESC-words */
+	    if (x1 > 14) {
+		int linbits_x1 = x1 - 15;
+		assert ( linbits_x1 <= h->linmax );
+		ext   |= linbits_x1 << 1;
+		xbits  = linbits;
+		x1     = 15;
+	    }
+
+	    if (x2 > 14) {
+		int linbits_x2 = x2 - 15;
+		assert ( linbits_x2 <= h->linmax );
+		ext  <<= linbits;
+		ext   |= linbits_x2;
+		xbits += linbits;
+		x2     = 15;
+	    }
+	    xlen = 16;
 	}
-	xlen = 16;
+
+	if (x2 != 0) {
+	    ext <<= 1;
+	    if (gi->xr[index+1] < 0)
+		ext++;
+	    cbits--;
+	}
+
+	assert ( (x1|x2) < 16u );
+
+	x1 = x1 * xlen + x2;
+	xbits -= cbits;
+	cbits += h->hlen  [x1];
+
+	assert ( cbits <= MAX_LENGTH );
+	assert ( xbits <= MAX_LENGTH );
+
+	putbits2(gfc, h->table [x1], cbits );
+	putbits2(gfc, ext,  xbits );
+	bits += cbits + xbits;
     }
-
-    if (x2 != 0) {
-	ext <<= 1;
-	if (gi->xr[index+1] < 0)
-	    ext++;
-	cbits--;
-    }
-
-    xbits -= cbits;
-
-    assert ( (x1|x2) < 16u );
-
-    x1 = x1 * xlen + x2;
-
-    cbits += h->hlen  [x1];
-
-    assert ( cbits <= MAX_LENGTH );
-    assert ( xbits <= MAX_LENGTH );
-
-    putbits2(gfc, h->table [x1], cbits );
-    putbits2(gfc, ext,  xbits );
-    
-    return cbits + xbits;
-}
-
-static int
-Huffmancodebits(lame_internal_flags *gfc, int tableindex, int start, int end, gr_info *gi)
-{
-    int i,bits;
-
-    assert(tableindex < 32);
-    if (!tableindex) return 0;
-
-    bits=0;
-    for (i = start; i < end; i += 2)
-      bits +=HuffmanCode(gfc, tableindex, i, gi);
-
     return bits;
 }
 
-
-
 /*
   Note the discussion of huffmancodebits() on pages 28
   and 29 of the IS, as well as the definitions of the side
@@ -578,8 +588,8 @@ ShortHuffmancodebits(lame_internal_flags *gfc, gr_info *gi)
 	region1Start = gi->big_values;
 
     /* short blocks do not have a region2 */
-    bits  = Huffmancodebits(gfc, gi->table_select[0], 0, region1Start, gi);
-    bits += Huffmancodebits(gfc, gi->table_select[1], region1Start, gi->big_values, gi);
+    bits  = Huffmancode(gfc, gi->table_select[0], 0, region1Start, gi);
+    bits += Huffmancode(gfc, gi->table_select[1], region1Start, gi->big_values, gi);
     return bits;
 }
 
@@ -605,16 +615,16 @@ LongHuffmancodebits(lame_internal_flags *gfc, gr_info *gi)
     if (region2Start > bigvalues)
 	region2Start = bigvalues;
 
-    bits  =Huffmancodebits(gfc, gi->table_select[0], 0, region1Start, gi);
-    bits +=Huffmancodebits(gfc, gi->table_select[1], region1Start, region2Start, gi);
-    bits +=Huffmancodebits(gfc, gi->table_select[2], region2Start, bigvalues, gi);
+    bits  =Huffmancode(gfc, gi->table_select[0], 0, region1Start, gi);
+    bits +=Huffmancode(gfc, gi->table_select[1], region1Start, region2Start, gi);
+    bits +=Huffmancode(gfc, gi->table_select[2], region2Start, bigvalues, gi);
     return bits;
 }
 
 inline static int
 writeMainData ( lame_global_flags * const gfp)
 {
-    int gr, ch, sfb,data_bits,scale_bits,tot_bits=0;
+    int gr, ch, sfb,data_bits,tot_bits=0;
     lame_internal_flags *gfc=gfp->internal_flags;
     III_side_info_t *l3_side;
 
@@ -627,52 +637,39 @@ writeMainData ( lame_global_flags * const gfp)
 		int slen1 = slen1_tab[gi->scalefac_compress];
 		int slen2 = slen2_tab[gi->scalefac_compress];
 		data_bits=0;
-		scale_bits=0;
-
 #ifdef DEBUG
 		hogege = gfc->bs.totbit;
 #endif
+		for (sfb = 0; sfb < gi->sfbdivide; sfb++) {
+		    if (gi->scalefac[sfb] == -1)
+			continue; /* scfsi is used */
+		    if (gi->scalefac[sfb] == -2)
+			gi->scalefac[sfb] = 0;
+		    putbits2(gfc, gi->scalefac[sfb], slen1);
+		    data_bits += slen1;
+		}
+		for (; sfb < gi->sfbmax; sfb++) {
+		    if (gi->scalefac[sfb] == -1)
+			continue; /* scfsi is used */
+		    if (gi->scalefac[sfb] == -2)
+			gi->scalefac[sfb] = 0;
+		    putbits2(gfc, gi->scalefac[sfb], slen2);
+		    data_bits += slen2;
+		}
+		assert(data_bits == gi->part2_length);
+
 		if (gi->block_type == SHORT_TYPE) {
-		    for (sfb = 0; sfb < SBPSY_s; sfb++) {
-			int slen = sfb < 6 ? slen1 : slen2;
-
-			assert(gi->scalefac.s[sfb][0]>=0);
-			assert(gi->scalefac.s[sfb][1]>=0);
-			assert(gi->scalefac.s[sfb][2]>=0);
-
-			putbits2(gfc, gi->scalefac.s[sfb][0], slen);
-			putbits2(gfc, gi->scalefac.s[sfb][1], slen);
-			putbits2(gfc, gi->scalefac.s[sfb][2], slen);
-			scale_bits += 3*slen;
-		    }
 		    data_bits += ShortHuffmancodebits(gfc, gi);
 		} else {
-		    int i;
-		    for (i = 0; i < sizeof(scfsi_band) / sizeof(int) - 1;
-			 i++) {
-			if (gr != 0 && l3_side->scfsi[ch][i])
-			    continue;
-
-			for (sfb = scfsi_band[i]; sfb < scfsi_band[i + 1];
-			     sfb++) {
-
-			    assert(gi->scalefac.l[sfb]>=0);
-			    putbits2(gfc, gi->scalefac.l[sfb],
-				    sfb < 11 ? slen1 : slen2);
-			    scale_bits += sfb < 11 ? slen1 : slen2;
-			}
-		    }
-		    data_bits +=LongHuffmancodebits(gfc, gi);
+		    data_bits += LongHuffmancodebits(gfc, gi);
 		}
-		data_bits +=huffman_coder_count1(gfc, gi);
+		data_bits += huffman_coder_count1(gfc, gi);
 #ifdef DEBUG
 		DEBUGF(gfc,"<%ld> ", gfc->bs.totbit-hogege);
 #endif
 		/* does bitcount in quantize.c agree with actual bit count?*/
-		assert(data_bits==gi->part2_3_length-gi->part2_length);
-		assert(scale_bits==gi->part2_length);
-		tot_bits += scale_bits + data_bits;
-
+		assert(data_bits == gi->part2_3_length + gi->part2_length);
+		tot_bits += data_bits;
 	    } /* for ch */
 	} /* for gr */
     } else {
@@ -680,11 +677,12 @@ writeMainData ( lame_global_flags * const gfp)
 	gr = 0;
 	for (ch = 0; ch < gfc->channels_out; ch++) {
 	    gr_info *gi = &l3_side->tt[gr][ch];
-	    int i, sfb_partition;
+	    int i, sfb_partition, scale_bits=0;
 	    assert(gi->sfb_partition_table);
 	    data_bits = 0;
-	    scale_bits=0;
-
+#ifdef DEBUG
+	    hogege = gfc->bs.totbit;
+#endif
 	    sfb = 0;
 	    sfb_partition = 0;
 
@@ -693,9 +691,9 @@ writeMainData ( lame_global_flags * const gfp)
 		    int sfbs = gi->sfb_partition_table[sfb_partition] / 3;
 		    int slen = gi->slen[sfb_partition];
 		    for (i = 0; i < sfbs; i++, sfb++) {
-			putbits2(gfc, Max(gi->scalefac.s[sfb][0], 0U), slen);
-			putbits2(gfc, Max(gi->scalefac.s[sfb][1], 0U), slen);
-			putbits2(gfc, Max(gi->scalefac.s[sfb][2], 0U), slen);
+			putbits2(gfc, Max(gi->scalefac[sfb*3+0], 0U), slen);
+			putbits2(gfc, Max(gi->scalefac[sfb*3+1], 0U), slen);
+			putbits2(gfc, Max(gi->scalefac[sfb*3+2], 0U), slen);
 			scale_bits += 3*slen;
 		    }
 		}
@@ -705,16 +703,18 @@ writeMainData ( lame_global_flags * const gfp)
 		    int sfbs = gi->sfb_partition_table[sfb_partition];
 		    int slen = gi->slen[sfb_partition];
 		    for (i = 0; i < sfbs; i++, sfb++) {
-			putbits2(gfc, Max(gi->scalefac.l[sfb], 0U), slen);
+			putbits2(gfc, Max(gi->scalefac[sfb], 0U), slen);
 			scale_bits += slen;
 		    }
 		}
 		data_bits +=LongHuffmancodebits(gfc, gi);
 	    }
 	    data_bits +=huffman_coder_count1(gfc, gi);
-
+#ifdef DEBUG
+	    DEBUGF(gfc,"<%ld> ", gfc->bs.totbit-hogege);
+#endif
 	    /* does bitcount in quantize.c agree with actual bit count?*/
-	    assert(data_bits==gi->part2_3_length-gi->part2_length);
+	    assert(data_bits==gi->part2_3_length);
 	    assert(scale_bits==gi->part2_length);
 	    tot_bits += scale_bits + data_bits;
 	} /* for ch */
@@ -744,7 +744,7 @@ compute_flushbits( const lame_global_flags * gfp, int *total_bytes_output )
 {
   lame_internal_flags *gfc=gfp->internal_flags;
   int flushbits,remaining_headers;
-  int bitsPerFrame, mean_bits;
+  int bitsPerFrame;
   int last_ptr,first_ptr;
   first_ptr=gfc->w_ptr;           /* first header to add to bitstream */
   last_ptr = gfc->h_ptr - 1;   /* last header to add to bitstream */
@@ -768,10 +768,10 @@ compute_flushbits( const lame_global_flags * gfp, int *total_bytes_output )
    * these bits are not necessary to decode the last frame, but
    * some decoders will ignore last frame if these bits are missing 
    */
-  getframebits(gfp,&bitsPerFrame,&mean_bits);
+  bitsPerFrame = getframebits(gfp);
   flushbits += bitsPerFrame;
   *total_bytes_output += bitsPerFrame;
-  // round up:  
+  /* round up:   */
   if (*total_bytes_output % 8) 
       *total_bytes_output = 1 + (*total_bytes_output/8);
   else
@@ -806,7 +806,6 @@ flush_bitstream(lame_global_flags *gfp)
   III_side_info_t *l3_side;
   int nbytes;
   int flushbits;
-  int bitsPerFrame, mean_bits;
   int last_ptr,first_ptr;
   first_ptr=gfc->w_ptr;           /* first header to add to bitstream */
   last_ptr = gfc->h_ptr - 1;   /* last header to add to bitstream */
@@ -815,11 +814,11 @@ flush_bitstream(lame_global_flags *gfp)
 
 
   if ((flushbits = compute_flushbits(gfp,&nbytes)) < 0) return;  
-  drain_into_ancillary(gfc, flushbits);
+  drain_into_ancillary(gfp, flushbits);
 
   /* check that the 100% of the last frame has been written to bitstream */
-  getframebits(gfp,&bitsPerFrame,&mean_bits);
-  assert (gfc->header[last_ptr].write_timing + bitsPerFrame  == gfc->bs.totbit);
+  assert (gfc->header[last_ptr].write_timing + getframebits(gfp)
+	  == gfc->bs.totbit);
 
   /* we have padded out all frames with ancillary data, which is the
      same as filling the bitreservoir with ancillary data, so : */
@@ -854,19 +853,21 @@ void  add_dummy_byte ( lame_global_flags* const gfp, unsigned char val )
   in the IS).
   */
 int
-format_bitstream(lame_global_flags *gfp, int bitsPerFrame)
+format_bitstream(lame_global_flags *gfp)
 {
     lame_internal_flags *gfc=gfp->internal_flags;
     int bits,nbytes;
     III_side_info_t *l3_side;
+    int bitsPerFrame;
     l3_side = &gfc->l3_side;
 
-    drain_into_ancillary(gfc, l3_side->resvDrain_pre);
+    bitsPerFrame = getframebits(gfp);
+    drain_into_ancillary(gfp, l3_side->resvDrain_pre);
 
     encodeSideInfo2(gfp,bitsPerFrame);
     bits = 8*gfc->sideinfo_len;
     bits+=writeMainData(gfp);
-    drain_into_ancillary(gfc, l3_side->resvDrain_post);
+    drain_into_ancillary(gfp, l3_side->resvDrain_post);
     bits += l3_side->resvDrain_post;
 
     l3_side->main_data_begin += (bitsPerFrame-bits)/8;
@@ -946,38 +947,59 @@ int copy_buffer(lame_internal_flags *gfc,unsigned char *buffer,int size,int mp3d
     
     if (mp3data) {
         UpdateMusicCRC(&gfc->nMusicCRC,buffer,minimum);
-#ifdef DECODE_ON_THE_FLY
-      untested code;
-      /* if, for somereason, we would like to decode the frame: */
-      {
-          short int pcm_out[2][1152];
+
+#ifdef DECODE_ON_THE_FLY 
+        if (gfc->decode_on_the_fly) {  /* decode the frame */
+          sample_t pcm_out[2][1152];
           int mp3out;
-          do {
-              /* re-synthesis to pcm.  Repeat until we get a mp3out=0 */
-              mp3out=lame_decode1(buffer,minimum,pcm_out[0],pcm_out[1]); 
-              /* mp3out = 0:  need more data to decode */
-              /* mp3out = -1:  error.  Lets assume 0 pcm output */
-              /* mp3out = number of samples output */
+          int i;
+
+          /* re-synthesis to pcm.  Repeat until we get a mp3out=0 */
+          mp3out=lame_decode1_unclipped(buffer,minimum,pcm_out[0],pcm_out[1]); 
+          /* mp3out = 0:  need more data to decode */
+          /* mp3out = -1:  error.  Lets assume 0 pcm output */
+          /* mp3out = number of samples output */
               
-              if (mp3out==-1) {
-                  // error decoding.  Not fatal, but might screw up are
-                  // ReplayVolume Info tag.  
-                  // what should we do?  ignore for now
-                  mp3out=0;
+          if (mp3out==-1) {
+              /* error decoding. Not fatal, but might screw up */
+              /* the ReplayGain tag. what should we do? ignore for now */
+              mp3out=0;
+          }
+          if (mp3out>0) {  
+            /* process the PCM data */
+
+   /*     if (mp3out>1152) {
+    *         this should not be possible, and indicates we have 
+    *         overflown the pcm_out buffer.  Fatal error. 
+    *       return -6;
+    *     } 
+    */
+
+            if (gfc->gfp->findPeakSample) {
+              for (i=0; i<mp3out; i++) {   
+                 if (pcm_out[0][i] > gfc->PeakSample)
+                   gfc->PeakSample = pcm_out[0][i];
+                 else if (-pcm_out[0][i] > gfc->PeakSample)
+                   gfc->PeakSample = -pcm_out[0][i]; 
               }
-              if (mp3out>0) {
-                  // process the PCM data.  
-                  if (mp3out>1152) {
-                      // this should not be possible, and indicates we have
-                      // overflowed the pcm_out buffer.  Fatal error.
-                      return -6;
-                  }
-              }
-              
-          } while (mp3out>0);
-      }
-#endif  
-    }
+              if (gfc->channels_out > 1)
+                for (i=0; i<mp3out; i++) {
+                  if (pcm_out[1][i] > gfc->PeakSample)
+                    gfc->PeakSample = pcm_out[1][i];
+                  else if (-pcm_out[1][i] > gfc->PeakSample)
+                    gfc->PeakSample = -pcm_out[1][i];
+                }
+            }
+
+            if (gfc->gfp->ReplayGain_decode)
+              if (AnalyzeSamples(pcm_out[0], pcm_out[1], mp3out, gfc->channels_out) == GAIN_ANALYSIS_ERROR)
+                 return -6;
+
+          } /* if (mp3out>0) */
+        } /* if (gfc->decode_on_the_fly) */ 
+#endif
+  
+    } /* if (mp3data) */
     return minimum;
 }
 
@@ -995,3 +1017,4 @@ void init_bit_stream_w(lame_internal_flags *gfc)
 }
 
 /* end of bitstream.c */
+
