@@ -41,17 +41,18 @@ struct lame_global_struct {
   /* general control params */
   int analysis;               /* collect data for a MP3 frame analyzer?      */
   int bWriteVbrTag;           /* add Xing VBR tag?                           */
-  int decode_only;            /* use lame/mpglib to convert mp3/ogg to wav   */
-  int ogg;                    /* encode to Vorbis .ogg file                  */
-
+  int decode_only;            /* use lame/mpglib to convert mp3 to wav       */
   int quality;                /* quality setting 0=best,  9=worst  default=5 */
   MPEG_mode mode;             /* see enum in lame.h
                                  default = LAME picks best value             */
   int mode_fixed;             /* ignored                                     */
-  int mode_automs;            /* use a m/s threshold based on compression
-                                 ratio                                       */
   int force_ms;               /* force M/S mode.  requires mode=1            */
   int free_format;            /* use free format? default=0                  */
+  int ReplayGain_input;       /* perform analysis on input data? default=0   */
+#ifdef DECODE_ON_THE_FLY
+  int ReplayGain_decode;      /* perfrom analysys on decoded data? default=0 */
+  int findPeakSample;         /* find peak sample? default=0                 */
+#endif
 
   /*
    * set either brate>0  or compression_ratio>0, LAME will compute
@@ -67,24 +68,20 @@ struct lame_global_struct {
   int original;                   /* mark as original. default=1            */
   int error_protection;           /* use 2 bytes per frame for a CRC
                                      checksum. default=0                    */
-  Padding_type padding_type;      /* PAD_NO = no padding,
-                                     PAD_ALL = always pad,
-                                     PAD_ADJUST = adjust padding,
-                                     default=2                              */
   int extension;                  /* the MP3 'private extension' bit.
                                      Meaningless                            */
   int strict_ISO;                 /* enforce ISO spec as much as possible   */
 
-  /* quantization/noise shaping */
   int disable_reservoir;          /* use bit reservoir?                     */
-  int experimentalX;            
+
+  /* quantization/noise shaping */
+  int quant_comp;
+  int quant_comp_short;
   int experimentalY;
-  int experimentalZ;
   int exp_nspsytune;
 
   double newmsfix;
   int preset_expopts;
-
   int preset;
 
   /* VBR control */
@@ -114,10 +111,15 @@ struct lame_global_struct {
    * psycho acoustics and other arguments which you should not change 
    * unless you know what you are doing
    */
+  int psymodel;                   /* select psychoacoustic model          */
+  int VBR_smooth;
+  float maskingadjust;
+  float maskingadjust_short;
   int ATHonly;                    /* only use ATH                         */
   int ATHshort;                   /* only use ATH for short blocks        */
   int noATH;                      /* disable ATH                          */
   int ATHtype;                    /* select ATH formula                   */
+  float ATHcurve;                 /* change ATH formula 4 shape           */
   float ATHlower;                 /* lower ATH by this many db            */
   int athaa_type;                 /* select ATH auto-adjust scheme        */
   int athaa_loudapprox;           /* select ATH auto-adjust loudness calc */
@@ -141,6 +143,9 @@ struct lame_global_struct {
   int   tune;               /* 0 off, 1 on */
   float tune_value_a;       /* used to pass values for debugging and stuff */
 
+  int   sparsing;
+  FLOAT sparse_low;
+  FLOAT sparse_high;
   
   struct {
     void (*msgf)  (const char *format, va_list ap);
@@ -163,17 +168,17 @@ struct lame_global_struct {
 
 
 
-  /****************************************************************************/
-  /* more internal variables, which will not exist after lame_encode_finish() */
-  /****************************************************************************/
+  /**************************************************************************/
+  /* more internal variables are stored in this structure:                  */
+  /**************************************************************************/
   lame_internal_flags *internal_flags;
 
-  /* VBR tags.  This data is here because VBR header is writen after
-   * input file is closed and *internal_flags struct is free'd */
+
+  /* VBR tag data */
   int TotalFrameSize;
-  //int* pVbrFrames;
   int nVbrNumFrames;
-  int nVbrFrameBufferSize;
+  /*int nVbrFrameBufferSize;*/
+  /*int* pVbrFrames; */
 
   struct {
     int integer[16];
@@ -190,3 +195,4 @@ struct lame_global_struct {
 } ;
 
 #endif /* LAME_GLOBAL_FLAGS_H */
+
