@@ -20,7 +20,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/* $Id: set_get.c,v 1.42 2002/10/16 18:01:10 bouvigne Exp $ */
+/* $Id: set_get.c,v 1.64 2003/12/03 01:44:49 olcios Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -35,10 +35,7 @@
 #endif
 
 
-
-extern int apply_preset(lame_global_flags*  gfp, int preset);
-
-
+#include "set_get.h"
 
 
 /*
@@ -249,7 +246,7 @@ lame_get_bWriteVbrTag( const lame_global_flags*  gfp )
 
 
 
-/* decode only, use lame/mpglib to convert mp3/ogg to wav */
+/* decode only, use lame/mpglib to convert mp3 to wav */
 int
 lame_set_decode_only( lame_global_flags*  gfp,
                       int                 decode_only )
@@ -277,29 +274,18 @@ lame_get_decode_only( const lame_global_flags*  gfp )
 
 
 /* encode a Vorbis .ogg file */
+/* DEPRECATED */
 int
 lame_set_ogg( lame_global_flags*  gfp,
               int                 ogg )
 {
-    /* default = 0 (disabled) */
-
-    /* enforce disable/enable meaning, if we need more than two values
-       we need to switch to an enum to have an apropriate representation
-       of the possible meanings of the value */
-    if ( 0 > ogg || 1 < ogg )
-        return -1;
-
-    gfp->ogg = ogg;
-
-    return 0;
+    return -1;
 }
 
 int
 lame_get_ogg( const lame_global_flags*  gfp )
 {
-    assert( 0 <= gfp->ogg && 1 >= gfp->ogg );
-
-    return gfp->ogg;
+    return 0;
 }
 
 
@@ -308,7 +294,7 @@ lame_get_ogg( const lame_global_flags*  gfp )
  * True quality is determined by the bitrate but this variable will effect
  * quality by selecting expensive or cheap algorithms.
  * quality=0..9.  0=best (very slow).  9=worst.  
- * recommended:  2     near-best quality, not too slow
+ * recommended:  3     near-best quality, not too slow
  *               5     good quality, fast
  *               7     ok quality, really fast
  */
@@ -353,6 +339,7 @@ lame_get_mode( const lame_global_flags*  gfp )
 
 
 /* Us a M/S mode with a switching threshold based on compression ratio */
+/* DEPRECATED */
 int
 lame_set_mode_automs( lame_global_flags*  gfp,
                       int                 mode_automs )
@@ -365,7 +352,7 @@ lame_set_mode_automs( lame_global_flags*  gfp,
     if ( 0 > mode_automs || 1 < mode_automs )
         return -1;
 
-    gfp->mode_automs = mode_automs;
+    lame_set_mode( gfp, JOINT_STEREO );
 
     return 0;
 }
@@ -373,9 +360,7 @@ lame_set_mode_automs( lame_global_flags*  gfp,
 int
 lame_get_mode_automs( const lame_global_flags*  gfp )
 {
-    assert( 0 <= gfp->mode_automs && 1 >= gfp->mode_automs );
-
-    return gfp->mode_automs;
+    return 1;
 }
 
 
@@ -436,6 +421,89 @@ lame_get_free_format( const lame_global_flags*  gfp )
 }
 
 
+/* Perform ReplayGain analysis on input data. */
+int
+lame_set_ReplayGain_input( lame_global_flags*  gfp,
+                           int                 ReplayGain_input )
+{
+    /* default = 0 (disabled) */
+
+    /* enforce disable/enable meaning, if we need more than two values
+       we need to switch to an enum to have an apropriate representation
+       of the possible meanings of the value */
+    if ( 0 > ReplayGain_input || 1 < ReplayGain_input )
+        return -1;
+
+    gfp->ReplayGain_input = ReplayGain_input;
+
+    return 0;
+}
+
+int
+lame_get_ReplayGain_input( const lame_global_flags*  gfp )
+{
+    assert( 0 <= gfp->ReplayGain_input && 1 >= gfp->ReplayGain_input );
+
+    return gfp->ReplayGain_input;
+}
+
+
+#ifdef DECODE_ON_THE_FLY
+/* Perform ReplayGain analysis on decoded data. */
+int
+lame_set_ReplayGain_decode( lame_global_flags*  gfp,
+                            int                 ReplayGain_decode )
+{
+    /* default = 0 (disabled) */
+
+    /* enforce disable/enable meaning, if we need more than two values
+       we need to switch to an enum to have an apropriate representation
+       of the possible meanings of the value */
+    if ( 0 > ReplayGain_decode || 1 < ReplayGain_decode )
+        return -1;
+
+    gfp->ReplayGain_decode = ReplayGain_decode;
+
+    return 0;
+}
+
+int
+lame_get_ReplayGain_decode( const lame_global_flags*  gfp )
+{
+    assert( 0 <= gfp->ReplayGain_decode && 1 >= gfp->ReplayGain_decode );
+
+    return gfp->ReplayGain_decode;
+}
+
+
+/* find peak sample. */
+int
+lame_set_findPeakSample( lame_global_flags*  gfp,
+                         int                 findPeakSample )
+{
+    /* default = 0 (disabled) */
+
+    /* enforce disable/enable meaning, if we need more than two values
+       we need to switch to an enum to have an apropriate representation
+       of the possible meanings of the value */
+    if ( 0 > findPeakSample || 1 < findPeakSample )
+        return -1;
+
+    gfp->findPeakSample = findPeakSample;
+
+    return 0;
+}
+
+int
+lame_get_findPeakSample( const lame_global_flags*  gfp )
+{
+    assert( 0 <= gfp->findPeakSample && 1 >= gfp->findPeakSample );
+
+    return gfp->findPeakSample;
+}
+#endif
+
+
 /* message handlers */
 int
 lame_set_errorf( lame_global_flags*  gfp,
@@ -607,22 +675,13 @@ int
 lame_set_padding_type( lame_global_flags*  gfp,
                        Padding_type        padding_type )
 {
-    /* default = 2 */
-
-    if ( 0 > padding_type || PAD_MAX_INDICATOR < padding_type )
-        return -1;  /* Unknown padding type */
-
-    gfp->padding_type = padding_type;
-
     return 0;
 }
 
 Padding_type
 lame_get_padding_type( const lame_global_flags*  gfp )
 {
-    assert( 0 <= gfp->padding_type && PAD_MAX_INDICATOR > gfp->padding_type );
-
-    return gfp->padding_type;
+    return PAD_ADJUST;
 }
 
 
@@ -715,12 +774,12 @@ lame_get_disable_reservoir( const lame_global_flags*  gfp )
 
 
 
-/* Select a different "best quantization" function. default = 0 */
 int
 lame_set_experimentalX( lame_global_flags*  gfp,
                         int                 experimentalX )
 {
-    gfp->experimentalX = experimentalX;
+    lame_set_quant_comp( gfp, experimentalX);
+    lame_set_quant_comp_short( gfp, experimentalX);
 
     return 0;
 }
@@ -728,7 +787,42 @@ lame_set_experimentalX( lame_global_flags*  gfp,
 int
 lame_get_experimentalX( const lame_global_flags*  gfp )
 {
-    return gfp->experimentalX;
+    return lame_get_quant_comp( gfp);
+}
+
+
+/* Select a different "best quantization" function. default = 0 */
+int
+lame_set_quant_comp( lame_global_flags*  gfp,
+                        int                 quant_type )
+{
+	gfp->quant_comp
+	= quant_type;
+
+    return 0;
+}
+
+int
+lame_get_quant_comp( const lame_global_flags*  gfp )
+{
+    return gfp->quant_comp;
+}
+
+
+/* Select a different "best quantization" function. default = 0 */
+int
+lame_set_quant_comp_short( lame_global_flags*  gfp,
+                        int                 quant_type )
+{
+	gfp->quant_comp_short = quant_type;
+
+    return 0;
+}
+
+int
+lame_get_quant_comp_short( const lame_global_flags*  gfp )
+{
+    return gfp->quant_comp_short;
 }
 
 
@@ -749,20 +843,17 @@ lame_get_experimentalY( const lame_global_flags*  gfp )
 }
 
 
-/* Another experimental option. For testing only. */
 int
 lame_set_experimentalZ( lame_global_flags*  gfp,
                         int                 experimentalZ )
 {
-    gfp->experimentalZ += experimentalZ;
-
     return 0;
 }
 
 int
 lame_get_experimentalZ( const lame_global_flags*  gfp )
 {
-    return gfp->experimentalZ;
+    return 0;
 }
 
 
@@ -825,7 +916,7 @@ lame_set_exp_nspsytune2_pointer( lame_global_flags*  gfp,
  * VBR control
  ***********************************************************************/
 
-// Types of VBR.  default = vbr_off = CBR
+/* Types of VBR.  default = vbr_off = CBR */
 int
 lame_set_VBR( lame_global_flags*  gfp,
               vbr_mode            VBR )
@@ -870,6 +961,7 @@ lame_set_VBR_q( lame_global_flags*  gfp,
         return -1;  /* Unknown VBR quality level! */
 
     gfp->VBR_q = VBR_q;
+//    lame_set_ATHcurve(gfp, VBR_q);
 
     return 0;
 }
@@ -1053,6 +1145,64 @@ lame_get_highpasswidth( const lame_global_flags*  gfp )
  * unless you know what you are doing
  */
 
+
+int
+lame_set_psy_model( lame_global_flags*  gfp,
+                   int               psy )
+{
+    gfp->psymodel = psy;
+    return 0;
+}
+
+int
+lame_get_psy_model( const lame_global_flags*  gfp )
+{
+    return gfp->psymodel;
+}
+
+int
+lame_set_vbr_smooth( lame_global_flags*  gfp,
+                   int               smooth )
+{
+    gfp->VBR_smooth = smooth;
+    return 0;
+}
+
+int
+lame_get_vbr_smooth( const lame_global_flags*  gfp )
+{
+    return gfp->VBR_smooth;
+}
+
+/* Adjust masking values. */
+int
+lame_set_maskingadjust( lame_global_flags*  gfp,
+                   float               adjust )
+{
+    gfp->maskingadjust = adjust;
+    return 0;
+}
+
+float
+lame_get_maskingadjust( const lame_global_flags*  gfp )
+{
+    return gfp->maskingadjust;
+}
+
+int
+lame_set_maskingadjust_short( lame_global_flags*  gfp,
+                   float               adjust )
+{
+    gfp->maskingadjust_short = adjust;
+    return 0;
+}
+
+float
+lame_get_maskingadjust_short( const lame_global_flags*  gfp )
+{
+    return gfp->maskingadjust_short;
+}
+
 /* Only use ATH for masking. */
 int
 lame_set_ATHonly( lame_global_flags*  gfp,
@@ -1122,19 +1272,36 @@ lame_get_ATHtype( const lame_global_flags*  gfp )
 }
 
 
+/* Select ATH formula 4 shape. */
+int
+lame_set_ATHcurve( lame_global_flags*  gfp,
+                   float ATHcurve )
+{
+    gfp->ATHcurve = ATHcurve;
+
+    return 0;
+}
+
+float
+lame_get_ATHcurve( const lame_global_flags*  gfp )
+{
+    return gfp->ATHcurve;
+}
+
+
 /* Lower ATH by this many db. */
 int
 lame_set_ATHlower( lame_global_flags*  gfp,
                    float               ATHlower )
 {
-    gfp->ATHlower = ATHlower;
+    gfp->ATHlower = -ATHlower / 10.0;
     return 0;
 }
 
 float
 lame_get_ATHlower( const lame_global_flags*  gfp )
 {
-    return gfp->ATHlower;
+    return -gfp->ATHlower * 10.0;
 }
 
 
@@ -1260,12 +1427,12 @@ lame_get_useTemporal( const lame_global_flags*  gfp )
 }
 
 
-/* Use temporal masking effect */
+/* Use inter-channel masking effect */
 int
 lame_set_interChRatio( lame_global_flags*  gfp,
 			float               ratio )
 {
-    /* default = 0.0 (no inter-cahnnel maskin) */
+    /* default = 0.0 (no inter-channel maskin) */
     if (! (0 <= ratio && ratio <= 1.0))
         return -1;
 
@@ -1277,7 +1444,8 @@ lame_set_interChRatio( lame_global_flags*  gfp,
 float
 lame_get_interChRatio( const lame_global_flags*  gfp )
 {
-    assert( 0 <= gfp->interChRatio && gfp->interChRatio <= 1.0);
+    assert( (0 <= gfp->interChRatio && gfp->interChRatio <= 1.0) ||
+        (gfp->interChRatio == -1));
 
     return gfp->interChRatio;
 }
@@ -1289,8 +1457,8 @@ lame_set_substep( lame_global_flags*  gfp,
 		  int                 method )
 {
     lame_internal_flags *gfc = gfp->internal_flags;
-    /* default = 0.0 (no inter-cahnnel maskin) */
-    if (! (0 <= method && method <= 2))
+    /* default = 0.0 (no substep noise shaping) */
+    if (! (0 <= method && method <= 7))
         return -1;
 
     gfc->substep_shaping = method;
@@ -1301,10 +1469,43 @@ int
 lame_get_substep(const lame_global_flags*  gfp )
 {
     lame_internal_flags *gfc = gfp->internal_flags;
-    assert(0 <= gfc->substep_shaping && gfc->substep_shaping <= 2);
+    assert(0 <= gfc->substep_shaping && gfc->substep_shaping <= 7);
     return gfc->substep_shaping;
 }
 
+/* scalefactors scale */
+int
+lame_set_sfscale( lame_global_flags* gfp,
+                int val )
+{
+    if (val)
+        gfp->internal_flags->noise_shaping = 2;
+    else
+        gfp->internal_flags->noise_shaping = 1;
+    return 0;
+}
+
+int
+lame_get_sfscale(const lame_global_flags*  gfp )
+{
+    return (gfp->internal_flags->noise_shaping == 2);
+}
+
+/* subblock gain */
+int
+lame_set_subblock_gain(lame_global_flags *gfp, int sbgain)
+{
+    gfp->internal_flags->subblock_gain = sbgain;
+    return sbgain;
+}
+
+int
+lame_get_subblock_gain(const lame_global_flags * gfp)
+{
+    return gfp->internal_flags->subblock_gain;
+}
+
+
 /* Disable short blocks. */
 int
 lame_set_no_short_blocks( lame_global_flags*  gfp,
@@ -1321,6 +1522,7 @@ lame_set_no_short_blocks( lame_global_flags*  gfp,
 
     return 0;
 }
+
 int
 lame_get_no_short_blocks( const lame_global_flags*  gfp )
 {
@@ -1366,6 +1568,47 @@ lame_get_force_short_blocks( const lame_global_flags*  gfp )
     }
 }
 
+int
+lame_set_short_threshold_lrm( lame_global_flags*  gfp,
+			  float lrm)
+{
+    lame_internal_flags *gfc = gfp->internal_flags;
+    gfc->nsPsy.attackthre = lrm;
+    return 0;
+}
+
+float
+lame_get_short_threshold_lrm( const lame_global_flags*  gfp)
+{
+    lame_internal_flags *gfc = gfp->internal_flags;
+    return gfc->nsPsy.attackthre;
+}
+
+int
+lame_set_short_threshold_s( lame_global_flags*  gfp,
+			  float s)
+{
+    lame_internal_flags *gfc = gfp->internal_flags;
+    gfc->nsPsy.attackthre_s = s;
+    return 0;
+}
+
+float
+lame_get_short_threshold_s( const lame_global_flags*  gfp)
+{
+    lame_internal_flags *gfc = gfp->internal_flags;
+    return gfc->nsPsy.attackthre_s;
+}
+
+int
+lame_set_short_threshold( lame_global_flags*  gfp,
+			  float lrm, float s)
+{
+    lame_set_short_threshold_lrm(gfp, lrm);    
+    lame_set_short_threshold_s(gfp, s);    
+    return 0;
+}
+
 
 /*
  * Input PCM is emphased PCM
@@ -1453,7 +1696,6 @@ lame_get_mf_samples_to_encode( const lame_global_flags*  gfp )
     return gfc->mf_samples_to_encode;
 }
 
-
 int CDECL lame_get_size_mp3buffer( const lame_global_flags*  gfp )
 {
     int size;
@@ -1461,6 +1703,35 @@ int CDECL lame_get_size_mp3buffer( const lame_global_flags*  gfp )
     return size;
 }
 
+int
+lame_get_RadioGain( const lame_global_flags*  gfp )
+{
+    lame_internal_flags *gfc = gfp->internal_flags;
+    return gfc->RadioGain;
+}
+
+int
+lame_get_AudiophileGain( const lame_global_flags*  gfp )
+{
+    lame_internal_flags *gfc = gfp->internal_flags;
+    return gfc->AudiophileGain;
+}
+
+#ifdef DECODE_ON_THE_FLY
+float
+lame_get_PeakSample( const lame_global_flags*  gfp )
+{
+    lame_internal_flags *gfc = gfp->internal_flags;
+    return (float)gfc->PeakSample;
+}
+
+int
+lame_get_decode_on_the_fly( const lame_global_flags* gfp )
+{
+    lame_internal_flags *gfc = gfp->internal_flags;
+    return gfc->decode_on_the_fly; 
+}
+#endif
 
 
 /*
@@ -1477,156 +1748,21 @@ lame_get_totalframes( const lame_global_flags*  gfp )
               ((double)gfp->in_samplerate * gfp->framesize);
 
     /* check to see if we underestimated totalframes */
-    //    if (totalframes < gfp->frameNum)
-    //        totalframes = gfp->frameNum;
+    /*    if (totalframes < gfp->frameNum) */
+    /*        totalframes = gfp->frameNum; */
 
     return totalframes;
 }
 
 
-/*
 
-UNDOCUMENTED, experimental settings.  These routines are not prototyped
-in lame.h.  You should not use them, they are experimental and may
-change.  
-
-*/
-
-
-/*
- *  just another daily changing developer switch  
- */
-void lame_set_tune( lame_global_flags* gfp, float val ) 
-{
-    gfp->tune_value_a = val;
-    gfp->tune = 1;
-}
-
-/* Custom msfix hack */
-void
-lame_set_msfix( lame_global_flags*  gfp, double msfix )
-{
-    /* default = 0 */
-    gfp->msfix = msfix;
-
-}
-
-int
-lame_set_preset_expopts( lame_global_flags*  gfp, int preset_expopts )
-{
-
-    lame_internal_flags *gfc = gfp->internal_flags;
-
-    gfc->presetTune.use = 1;
-
-    /* default = 0 (disabled) */
-    gfp->preset_expopts = preset_expopts;
-
-    switch (preset_expopts)
-    {
-        case 1:
-
-          lame_set_exp_nspsytune(gfp, lame_get_exp_nspsytune(gfp) | 1);
-          lame_set_experimentalX(gfp, 3);
-          lame_set_exp_nspsytune(gfp, lame_get_exp_nspsytune(gfp) | 2); // safejoint
-          lame_set_ATHtype(gfp, 2);
-
-          gfc->presetTune.attackthre   = 35;
-          gfc->presetTune.attackthre_s = 150;
-          gfc->presetTune.ms_maskadjust = .5;
-		  gfc->presetTune.quantcomp_type_s = 3;
-          gfc->presetTune.quantcomp_alt_type = 3;
-          gfc->presetTune.athadjust_switch_level = 2; // Always switch
-
-          break;
-
-        case 2:
-
-          if (gfp->VBR == vbr_mtrh) {
-             lame_set_experimentalX(gfp, 2);
-             gfc->presetTune.quantcomp_adjust_mtrh = 9;
-             gfc->presetTune.quantcomp_type_s = 4;
-             gfc->presetTune.quantcomp_alt_type = 0;
-             gfc->presetTune.athadjust_safe_noiseshaping_thre = 0.0;
-			 gfc->presetTune.athadjust_safe_athaasensitivity = 8.0;
-          }
-          else {
-             lame_set_experimentalX(gfp, 3);
-             gfc->presetTune.quantcomp_adjust_rh_tot = 600;
-			 gfc->presetTune.quantcomp_adjust_rh_max = 60;
-             gfc->presetTune.quantcomp_type_s = 3;
-             gfc->presetTune.quantcomp_alt_type = 1;
-          }
-
-          lame_set_exp_nspsytune(gfp, lame_get_exp_nspsytune(gfp) | 1);
-          lame_set_experimentalZ(gfp, 1);
-          lame_set_VBR_q(gfp, 2);
-          lame_set_exp_nspsytune(gfp, lame_get_exp_nspsytune(gfp) | 2); // safejoint
-          lame_set_ATHtype(gfp, 2);				
-          // modify sfb21 by 3 dB plus ns-treble=0                 
-          lame_set_exp_nspsytune(gfp, lame_get_exp_nspsytune(gfp) | (12 << 20));
-
-          gfc->presetTune.attackthre   = 35;
-          gfc->presetTune.attackthre_s = 150;
-          gfc->presetTune.ms_maskadjust = .5;
-          gfc->presetTune.athadjust_switch_level = 1;
-          gfc->presetTune.athadjust_msfix = 2.13;
-
-          break;
-
-        case 3:
-
-          if (gfp->VBR == vbr_mtrh) {
-             gfc->presetTune.quantcomp_type_s = 4;
-             gfc->presetTune.quantcomp_adjust_mtrh = 9;
-			 gfc->presetTune.quantcomp_alt_type = 0;
-             (void) lame_set_ATHlower( gfp, -2 );
-             gfc->presetTune.athadjust_safe_noiseshaping_thre = 0.0;
-			 gfc->presetTune.athadjust_safe_athaasensitivity = 8.0;
-          }
-          else {
-             gfc->presetTune.quantcomp_type_s = 3;
-             gfc->presetTune.quantcomp_adjust_rh_tot = 600;
-			 gfc->presetTune.quantcomp_adjust_rh_max = 60;
-             (void) lame_set_ATHlower( gfp, -1 );
-          }
-
-          lame_set_exp_nspsytune(gfp, lame_get_exp_nspsytune(gfp) | 1);
-          lame_set_experimentalZ(gfp, 1);
-          lame_set_experimentalX(gfp, 1);
-          lame_set_VBR_q(gfp, 2);
-          lame_set_exp_nspsytune(gfp, lame_get_exp_nspsytune(gfp) | 2); // safejoint
-   (void) lame_set_msfix( gfp, 2.13 );
-          lame_set_ATHtype(gfp, 4);
-          // modify sfb21 by 3.75 dB plus ns-treble=0                 
-          lame_set_exp_nspsytune(gfp, lame_get_exp_nspsytune(gfp) | (15 << 20));
-
-          gfc->presetTune.attackthre   = 35;
-          gfc->presetTune.attackthre_s = 150;
-          gfc->presetTune.ms_maskadjust = .5;
-          gfc->presetTune.athadjust_switch_level = 1;
-
-          break;
-    }
-    return 0;
-}
-
-int
-lame_set_preset_notune( lame_global_flags*  gfp, int preset_notune )
-{
-    lame_internal_flags *gfc = gfp->internal_flags;
-
-    gfc->presetTune.use = 0;  // Turn off specialized preset tunings
-
-    return 0;
-}
 
 
 int
 lame_set_preset( lame_global_flags*  gfp, int preset )
 {
     gfp->preset = preset;
-    return apply_preset(gfp, preset);
+    return apply_preset(gfp, preset, 1);
 }
 
 
@@ -1650,21 +1786,86 @@ lame_set_asm_optimizations( lame_global_flags*  gfp, int optim, int mode)
         }
         default: return optim;
     }
-
 }
 
 
 
 
 
+/*
+
+UNDOCUMENTED, experimental settings.  These routines are not prototyped
+in lame.h.  You should not use them, they are experimental and may
+change.  
+
+*/
+
+
+/*
+ *  just another daily changing developer switch  
+ */
+void lame_set_tune( lame_global_flags* gfp, float val ) 
+{
+    gfp->tune_value_a = val;
+    gfp->tune = 1;
+}
+
+void lame_set_ms_sparsing( lame_global_flags* gfp, int val )
+{
+    gfp->sparsing = val;
+}
+
+int lame_get_ms_sparsing( lame_global_flags* gfp )
+{
+    return gfp->sparsing;
+}
+
+void lame_set_ms_sparse_low( lame_global_flags* gfp, float val )
+{
+    gfp->sparse_low = val;
+}
+
+float lame_get_ms_sparse_low( lame_global_flags* gfp )
+{
+    return gfp->sparse_low;
+}
+
+void lame_set_ms_sparse_high( lame_global_flags* gfp, float val )
+{
+    gfp->sparse_high = val;
+}
+
+float lame_get_ms_sparse_high( lame_global_flags* gfp )
+{
+    return gfp->sparse_high;
+}
 
 
 
+/* Custom msfix hack */
+void
+lame_set_msfix( lame_global_flags*  gfp, double msfix )
+{
+    /* default = 0 */
+    gfp->msfix = msfix;
+}
 
+float
+lame_get_msfix( const lame_global_flags*  gfp )
+{
+    return gfp->msfix;
+}
 
+int
+lame_set_preset_expopts( lame_global_flags*  gfp, int preset_expopts )
+{
 
+    return 0;
+}
 
-
-
-
+int
+lame_set_preset_notune( lame_global_flags*  gfp, int preset_notune )
+{
+    return 0;
+}
 
